﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Text;

using HeuristicLab.Services.Authentication.DataAccess;
using HeuristicLab.Services.Authentication.DataTransfer;
using HeuristicLab.Services.Authentication;
using DA = HeuristicLab.Services.Authentication.DataAccess;
using DT = HeuristicLab.Services.Authentication.DataTransfer;
using HLSA = HeuristicLab.Services.Authentication;

namespace HeuristicLab.Services.Authentication.AdminMethods {
  public class ClientAdmin : ResourceAdmin {

    private void TransferData(DT.Client c, Hashtable values) {
      c.ResourceID = new Guid((string)values[eResource.ResourceID]);
      c.Name = (string)values[eResource.Name];
      c.Description = (string)values[eResource.Description];
      c.ResourceType = "Client";
      c.ProcessorType = (string)values[eResource.ProcessorType];
      c.NumberOfProcessors = (string)values[eResource.NumberOfProcessors];
      c.NumberOfThreads = (string)values[eResource.NumberOfThreads];
      c.MemorySize = (string)values[eResource.MemorySize];
      c.IPAdress = (string)values[eResource.IPAdress];
      c.OperatingSystem = (string)values[eResource.OperatingSystem];
    }

    public override HashSet<Hashtable> Get(Hashtable values) {
      values[eResource.ResourceType] = "Client";
      String QueryString = "SELECT * FROM Resource";
      List<String> lcv = new List<String>();
      String QueryStringExtension = " WHERE ";
      CreateQueryStringExtension(lcv, ref QueryStringExtension, values);
      if (QueryStringExtension.CompareTo(" WHERE ") > 0) {
        QueryString += QueryStringExtension;
      } // else select all
      //string val1 = "%" + ((string)tmpHt[eClient.Name]) + "%";
      //QueryString += eClient.Name + " LIKE {0}";
      object[] compValues = lcv.ToArray();
      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {
        if (cmdc.DatabaseExists()) {
          Table<DA.Resource> clTable = cmdc.GetTable<DA.Resource>();
          IEnumerable<DA.Resource> clients = cmdc.ExecuteQuery<DA.Resource>(QueryString, compValues);
          HashSet<Hashtable> resultSet = new HashSet<Hashtable>();
          var selectedClients = from c in clients orderby c.Name select HLSA.Convert.ToDto((DA.Client)c);
          foreach (DT.Client cl in selectedClients) {
            Hashtable ht = new Hashtable();
            ht.Add(eResource.ResourceID, cl.ResourceID);
            ht.Add(eResource.Name, cl.Name);
            ht.Add(eResource.Description, cl.Description);
            ht.Add(eResource.ResourceType, cl.ResourceType);
            ht.Add(eResource.ProcessorType, cl.ProcessorType);
            ht.Add(eResource.NumberOfProcessors, cl.NumberOfProcessors);
            ht.Add(eResource.NumberOfThreads, cl.NumberOfThreads);
            ht.Add(eResource.IPAdress, cl.IPAdress);
            ht.Add(eResource.MemorySize, cl.MemorySize);
            ht.Add(eResource.OperatingSystem, cl.OperatingSystem);
            // look for group entries in ResourceResourcGroup  
            //Guid groupID = new Guid();  /// ??
            String groupID = "";
            String groupName = "";
            GetGroupIDandName(ref groupID, ref groupName, cmdc, cl.ResourceID);
            ht.Add(eResource.ResourceIDGroup, groupID);
            ht.Add(eResource.ResourceNameGroup, groupName);

            resultSet.Add(ht);
          }
          return resultSet;
        }
        return null;
      }
    }
    
    public override bool Create(Hashtable values) {
      if (((string)values[eResource.ResourceIDGroup] == "") ||
          ((string)values[eResource.ResourceID] != "")) {
        return false;
      }
      DT.Client c = new DT.Client();
      values[eResource.ResourceID] = Guid.NewGuid().ToString();
      values[eResource.ResourceType] = "Client";
      TransferData(c, values);
      Guid groupID = new Guid((string)values[eResource.ResourceIDGroup]);
      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {
        if (cmdc.DatabaseExists()) {
          // Client
          DA.Client clEntity = HLSA.Convert.ToEntity(c);
          cmdc.GetTable<DA.Resource>().InsertOnSubmit(clEntity);

          Table<DA.Resource> clTable = cmdc.GetTable<DA.Resource>();
          //Table<ResourceResourceGroup> rrgTable = cmdc.GetTable<ResourceResourceGroup>();
          DA.Resource groupEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == groupID);
          if (groupEntity == null)
            return false;

          cmdc.SubmitChanges();

          DA.Resource clientEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == c.ResourceID);
          if (clientEntity == null)
           return false;

          //ResourceResourceGroup 
          ResourceResourceGroup rrgEntity = new ResourceResourceGroup();
          rrgEntity.ResourceGroupID = groupEntity.ResourceID;
          rrgEntity.ResourceID = clientEntity.ResourceID;
          cmdc.GetTable<ResourceResourceGroup>().InsertOnSubmit(rrgEntity);

          cmdc.SubmitChanges();
          return true;
        }
        return false;
      }
    }

    public override bool Update(Hashtable values) {
      string groupIDString = (string)values[eResource.ResourceIDGroup];
      string IDString = (string)values[eResource.ResourceID];

      DT.Client uc = new DT.Client();
      bool groupModified = false;
      bool success = true;
      bool validGroupID = (IDString != "") && (IsIDString(groupIDString));
      bool validID = (IDString != "") && (IsIDString(IDString));

      if (!validGroupID || !validID || (IDString.CompareTo(groupIDString) == 0))
        return false;

      Guid resID = new Guid(IDString);
      TransferData(uc, values);

      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {

        if (cmdc.DatabaseExists()) {
            Guid currentGroupID = new Guid(groupIDString);
            Guid oldGroupID = new Guid();

            ResourceResourceGroup rrgEntity = cmdc.GetTable<ResourceResourceGroup>().FirstOrDefault(
                                 x => x.ResourceID == resID);
          if (rrgEntity != null) {
            oldGroupID = rrgEntity.ResourceGroupID;
          }
          // check if groupname was modified
          groupModified = (oldGroupID != currentGroupID);

          DA.Resource clientEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == uc.ResourceID);
          // update client
          if ((clientEntity != null) && success) {
            uc.ResourceType = clientEntity.ResourceType; // dont allow change of ResourceTyp
            HLSA.Convert.ToEntity(uc, (DA.Client)clientEntity);
            if (groupModified) {
              // remove entry in ResourceResourcGroup if group modiied
              RemoveRRGEntry(cmdc, resID);
            }
            cmdc.SubmitChanges();
          }

          if (groupModified && success) {
            CreateRRGEntry(cmdc, resID, currentGroupID);
            cmdc.SubmitChanges();
          }
        }
      }
      return success;
    }    

  }
}
