﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Text;

using HeuristicLab.Services.Authentication.DataAccess;
using HeuristicLab.Services.Authentication.DataTransfer;
using HeuristicLab.Services.Authentication;
using DA = HeuristicLab.Services.Authentication.DataAccess;
using DT = HeuristicLab.Services.Authentication.DataTransfer;
using HLSA = HeuristicLab.Services.Authentication;

namespace HeuristicLab.Services.Authentication.AdminMethods {

  public enum eResource {
    ResourceID = 1, Name = 2, Description = 3, ResourceType = 4,
    ProcessorType = 5, NumberOfProcessors = 6, NumberOfThreads = 7,
    MemorySize = 8, IPAdress = 9, OperatingSystem = 10, ResourceNameGroup, ResourceIDGroup,
    GroupGroupID, GroupGroupName
  };

  public abstract class ResourceAdmin {    

    public abstract HashSet<Hashtable> Get(Hashtable values);
    public abstract bool Create(Hashtable values);    
    public abstract bool Update(Hashtable values);    
    
    public bool RemoveRRGEntry(ClientManagmentDataContext cmdc, Guid resID) {
      ResourceResourceGroup rrgEntity = cmdc.GetTable<ResourceResourceGroup>().FirstOrDefault(
                      x => x.ResourceID == resID);
      //ResourceResourceGroup rrgEntityGroup = cmdc.GetTable<ResourceResourceGroup>().FirstOrDefault(
      //                x => x.ResourceGroupID == resID);
      if (rrgEntity != null) { // && (rrgEntityGroup == null)) {
        cmdc.GetTable<ResourceResourceGroup>().DeleteOnSubmit(rrgEntity);
        return true;
      }
      return false;
    }
    
    public void CreateRRGEntry(ClientManagmentDataContext cmdc, Guid clientID, Guid groupID) {
      ResourceResourceGroup rrgEntity = new ResourceResourceGroup();
      rrgEntity.ResourceGroupID = groupID;
      rrgEntity.ResourceID = clientID;
      cmdc.GetTable<ResourceResourceGroup>().InsertOnSubmit(rrgEntity);
    }
    
    public void GetGroupIDandName(ref String groupID, ref String groupName, ClientManagmentDataContext cmdc, Guid clientID) {
      ResourceResourceGroup rrgEntity = cmdc.GetTable<ResourceResourceGroup>().FirstOrDefault(
                               x => x.ResourceID == clientID);
      if (rrgEntity != null) {
        groupID = (rrgEntity.ResourceGroupID).ToString();
        DA.Resource groupEntity = cmdc.GetTable<DA.Resource>().FirstOrDefault(
                  x => x.ResourceID == rrgEntity.ResourceGroupID);
        groupName = groupEntity.Name;
      }
    }
    
    public void CreateQueryStringExtension(List<String> compareValues, ref String QueryStringExtension, Hashtable ht) {
      int i = 0;
      foreach (DictionaryEntry kvp in ht) {
        if (((string)kvp.Value != "") &&
          (kvp.Key.Equals(eResource.ResourceNameGroup) == false) &&
          (kvp.Key.Equals(eResource.ResourceIDGroup) == false)) {
          compareValues.Add(String.Format("%" + (ht[kvp.Key]).ToString() + "%"));
          if (i == 0)
            QueryStringExtension += (kvp.Key.ToString() + " LIKE {" + i + "}");
          else
            QueryStringExtension += " AND " + (kvp.Key.ToString() + " LIKE {" + i + "}");
          i++;
        }
      }
    }
    
    public bool Delete(Hashtable values) {
      string idStr = (string)values[eResource.ResourceID];
      if ((idStr == "") || (!IsIDString(idStr))) {
        return false;
      }

      using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext()) {
        Guid clientID = new Guid(idStr);

        if (cmdc.DatabaseExists()) {
          // remove entry in ResourceResourcGroup  
          //if (!(RemoveRRGEntry(cmdc, clientID)))
          //  return false;
          RemoveRRGEntry(cmdc, clientID);
          // check if resource is member of a group
          DA.Resource Entity = cmdc.GetTable<DA.Resource>().FirstOrDefault(x => x.ResourceID == new Guid(idStr));
          
          // only allow removing if group is not still in use
          DA.ResourceResourceGroup rrgEntity = cmdc.GetTable<DA.ResourceResourceGroup>()
                              .FirstOrDefault(x => x.ResourceGroupID == new Guid(idStr));
          if ((Entity != null) && (rrgEntity == null)) {
            cmdc.GetTable<DA.Resource>().DeleteOnSubmit(Entity);
            cmdc.SubmitChanges();
            return true;
          }
        }
      } return false;
    }

    public bool IsIDString (string idString){
      try{
        Guid gid = new Guid(idString);
        return true;
      }
      catch (Exception e) {return false;}
    }

  }
}
